Ext.define('DSS.addon.SelectPreset', {
	extend : 'DSS.component.Base',
	mixins : {
		observable : 'Ext.util.Observable'
	},
	getHeight : function() {
		return this.iDefaultHeight;
	},

	height : '100%',
	grids : {},
	selectedRoomID : -1,
	presetID : -1,
	macPanel : null,
	arrayAllPresets : null,
	iDefaultWidth : 150,
	iDefaultHeight : 332,
	defMacGridRenderer : function(val, metadata, record, colIdx, store, view) {
		if (record.get('name') == '-') {
        	return '<table style="height:16px;width:100%;"><tr style="height:16px;"><td width="100%"><hr></td></tr></table>';
        } 
		var fIsPassive = false;
		if (record.get('passive') != null)
			if (record.get('passive'))
				fIsPassive = true;
		var fHasNext = false;
		if (record.get('next') == true) {
			fHasNext = true;
		}
		var fHasIcon = false;
		if (record.get('icon') != null) {
			fHasIcon = true;
		}

		var sReturn = '<table style="height:16px;"><tr>';
		var iColspaceCounter = 3;
		if (fHasIcon) {
			iColspaceCounter--;
			sReturn += '<td><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
		}
		if (fIsPassive) {
			sReturn += '<td style="color:#888888;" colspan="' + iColspaceCounter + '" width="100%">' + record.get('name') + '</td>';
		} else {
			sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + record.get('name') + '</td>';
		}
		if (fHasNext) {
			sReturn += '<td>&gt;</td>';
		}

		return sReturn + '</tr></table>';
	},
	isPresetUsed: function(zone,scene) {
		if (this.presetID==scene)
			if (this.selectedRoomID==zone)
				return false;
		for (var iIndex=0;iIndex<presetStore.count();iIndex++) {
			var oTrigger=presetStore.getAt(iIndex).get('triggers');
			for (var jIndex=0;jIndex<oTrigger.length;jIndex++) {
				if (oTrigger[jIndex].zone==zone)
					if (oTrigger[jIndex].scene==scene)
						return true;
			}
		}
		return false;
	},
	constructGridLvlPresets : function() {
				
		if (Ext.data.StoreManager.lookup('selectpreset_list') != null) {
			Ext.data.StoreManager.lookup('selectpreset_list').removeAll();
		} else {
			Ext.create('Ext.data.Store', {
				storeId : 'selectpreset_list',
				fields : [ '_id', 'name', 'next', 'icon', 'passive' ],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		}

		var arrayPresets = [];
		var sceneArray=[17,18,19,-1,20,21,22,-1,23,24,25,-1,26,27,28,-1,29,30,31];
		
		for (var iIndex=0;iIndex<sceneArray.length;iIndex++) {
			if (sceneArray[iIndex]!=-1)
				arrayPresets.push(new Object({
					name : dss.staticDataModel.getDefaultNameForScene(this.selectedRoomID, 4, sceneArray[iIndex]),
					_id : sceneArray[iIndex],
					icon : '/framework/images/groupIcons/group_4_active.png',
					passive : this.isPresetUsed(this.selectedRoomID,sceneArray[iIndex]),
					next : false
				}));
			else
				arrayPresets.push(new Object({
					name : '-',
					_id : -1,
					icon : null,
					passive : true,
					next : false
				}));

		}

		Ext.data.StoreManager.lookup('selectpreset_list').add(arrayPresets);

		this.grids.lvlPresets = Ext.create('Ext.grid.Panel', {
			hideHeaders : true,
			height : this.iDefaultHeight,
			width : 624,
			autoScroll : true,
			viewConfig : {
				stripeRows : true
			},
			selModel : {
				mode : 'SINGLE'
			},
			store : Ext.data.StoreManager.lookup('selectpreset_list'),
			columns : [ {
				sortable : false,
				hideable : false,
				dataIndex : 'name',
				flex : 1,
				renderer : this.defMacGridRenderer
			} ]
		});

		this.grids.lvlPresets.parent = this;

		this.grids.lvlPresets.on('itemclick', function(grid, record) {
			if (record.get('passive')) {
				this.getSelectionModel().deselect(record);
			}
		});
		this.macPanel.add(this.grids.lvlPresets);

	},
	constructMacGrids : function() {
		var me = this;
		this.grids.lvl1 = Ext.create('Ext.grid.Panel', {
			hideHeaders : true,
			height : this.iDefaultHeight,
			width : this.iDefaultWidth,
			autoScroll : true,
			viewConfig : {
				stripeRows : true
			},
			selModel : {
				mode : 'SINGLE'
			},
			columns : [ {
				sortable : false,
				hideable : false,
				dataIndex : 'name',
				flex : 1,
				renderer : this.defMacGridRenderer
			} ]
		});
		this.grids.lvl1.getSelectionModel().store = this.grids.lvl1.store;
		this.grids.lvl1.parent = this;

		this.grids.lvl1.on('itemclick', function(grid, record) {
			this.parent.macPanel.remove(this.parent.grids.lvlPresets);
			me.selectedRoomID = record.get('_id');
			this.parent.constructGridLvlPresets();
		});
	},
	constructor : function() {
		this.constructMacGrids();

		this.macPanel = Ext.create('Ext.panel.Panel', {
			border : '0',
			autoScroll : true,
			height : this.iDefaultHeight + 2,
			layout : {
				type : 'hbox'
			},
			items : [ this.grids.lvl1 ],
			scroll : 'horizontal'
		});

		Ext.apply(this, {
			items : [ this.macPanel ]
		});

		this.callParent(arguments);
	},
	reset : function() {
		this.selectedRoomID = -1;
		this.presetID = -1;
		this.grids.lvl1.reconfigure(Ext.data.StoreManager.lookup('selectpreset_l1'));
		this.macPanel.remove(this.grids.lvlPresets);
		this.grids.lvl1.getSelectionModel().deselectAll();
	},
	setValue : function(values) {
		this.grids.lvl1.reconfigure(Ext.data.StoreManager.lookup('selectpreset_l1'));
		this.macPanel.remove(this.grids.lvlPresets);

		this.selectedRoomID = values[0].zone;
		this.presetID = values[0].scene;

		var store = this.grids.lvl1.getStore();
		var foundRec = store.getAt(store.findExact('_id', "" + this.selectedRoomID));
		this.grids.lvl1.getSelectionModel().select(foundRec);
	
		this.constructGridLvlPresets();
		var oRecordZone2 = this.grids.lvlPresets.store.find('_id', this.presetID, 0, false, false, true);
		if (oRecordZone2 != -1)
			this.grids.lvlPresets.getSelectionModel().select(oRecordZone2);
	},
	isValid : function() {
		var bValid = false;

		var oSelectionLVL1 = this.grids.lvl1.getSelectionModel().getSelection();
		if (this.grids.lvlPresets != null) {
			var oSelectionPreset = this.grids.lvlPresets.getSelectionModel().getSelection();
			if (oSelectionLVL1[0] != null && oSelectionPreset[0] != null)
				bValid = true;
		}
		return bValid;
	},
	getValue : function() {
		var returnValue = [];
		try {
			var oSelectionLVL1 = this.grids.lvl1.getSelectionModel().getSelection();
			if (this.grids.lvlPresets != null) {
				var oSelectionPlaylist = this.grids.lvlPresets.getSelectionModel().getSelection();
				var oObject={
					zone:oSelectionLVL1[0].get('_id'),
					group:4,
					type:'bus-zone-scene',
					scene:oSelectionPlaylist[0].get('_id')
				};
				returnValue.push(oObject);
			}
		} catch (e) {
		}
		return returnValue;
	}
});


Ext.define('DSS.addon.SetVolume', {
	extend : 'DSS.component.Base',
	mixins : {
		observable : 'Ext.util.Observable'
	},
	checkVolume : null,
	timeSpanVolume : null,
	constructor : function() {
		this.checkVolume = Ext.create('Ext.form.field.Checkbox', {
			fieldLabel : "",
			align : 'middle',
		});

		var lblSetVolume = Ext.create('Ext.form.Label', {
			text : _("Set volume to"),
			align : 'middle',
			cls : 'x-form-item-label',
			margin : '0 0 0 10'
		});

		this.timeSpanVolume = Ext.create('Ext.form.field.Number', {
			fieldLabel : "",
			margin : '0 0 0 10',
			width : 50,
			step : 1,
			value : 0,
			minValue : 0,
			maxValue : 100
		});

		var lblSetVolume2 = Ext.create('Ext.form.Label', {
			text : "%",
			align : 'middle',
			cls : 'x-form-item-label',
			margin : '0 0 0 4'
		});

		var rowVolume = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ this.checkVolume, lblSetVolume, this.timeSpanVolume, lblSetVolume2 ],
			layout : 'column',
			margin : '2 0 5 0',
			width : 530
		});

		Ext.apply(this, {
			items : [ rowVolume ]
		});
		this.callParent(arguments);
	},
	reset : function() {
		this.checkVolume.setValue(false);
		this.timeSpanVolume.setValue(0);
	},
	setValue : function(value) {
		var nSelectedPresetID = this.dssEditor.editControls.id.getValue();

		var sString = dss.ajaxSyncRequest('/json/property/query', {
			'query' : '/scripts/sonos-controller/sonos-presets/' + nSelectedPresetID + '(*)'
		});
		var result = Ext.JSON.decode(sString);
		var arrayVolume = dss.parsePropertyQueryAnswer(result, [ {
			name : 'volumeSet',
			type : 'simple'
		}, {
			name : 'volume',
			type : 'simple'
		} ]);

		this.checkVolume.setValue(false);
		this.timeSpanVolume.setValue(0);
		if (arrayVolume != null && JSON.parse(arrayVolume[0].volumeSet)) {
			this.checkVolume.setValue(true);
			this.timeSpanVolume.setValue(arrayVolume[0].volume);
		}
	},
	isValid : function() {
		return true;
	},
	getValue : function() {
		var returnValue = new Object();

		returnValue.setVolume = this.checkVolume.checked;
		returnValue.volume = this.timeSpanVolume.getValue();

		return returnValue;
	}
});

Ext.define('DSS.addon.ManualIP', {
	extend : 'DSS.component.Base',
	mixins : {
		observable : 'Ext.util.Observable'
	},
	getHeight : function() {
		return 20;
	},
	txtEnterIPParts : [],

	constructor : function(config) {
		Ext.apply(Ext.form.field.VTypes, {
			IPAddress : function(v, field) {
				if (v == "")
					return false;
				var partValue = parseInt(v);
				return (partValue < 255);
			},
			IPAddressText : _('Not valid IP address part'),
			IPAddressMask : /[\d]/i
		});

		var lblEnterIP = Ext.create('Ext.form.Label', {
			text : _("Manually enter IP address") + ":",
			align : 'left',
			cls : 'x-form-item-label',
			margin : '0 50 0 10'
		});

		var lblDots = [];
		for (var i = 0; i < 3; i++) {
			lblDots[i] = Ext.create('Ext.form.Label', {
				text : ".",
				align : 'left',
				cls : 'x-form-item-label',
				margin : '0 2 0 2'
			});
		}

		for (var i = 0; i < 4; i++) {
			this.txtEnterIPParts[i] = Ext.create('Ext.form.field.Text', {
				fieldLabel : "",
				labelWidth : 0,
				margin : '0 0 0 0',
				value : "",
				maxLength : 3,
				enforceMaxLength : true,
				vtype : 'radius',
				width : 30,
				vtype : 'IPAddress'
			});
		}

		var rowIpAddress = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ lblEnterIP, this.txtEnterIPParts[0], lblDots[0], this.txtEnterIPParts[1], lblDots[1], this.txtEnterIPParts[2], lblDots[2], this.txtEnterIPParts[3] ],
			layout : 'column',
			margin : '2 0 5 0',
			width : 530
		});

		Ext.apply(this, {
			items : [ rowIpAddress ]
		});

		this.callParent(arguments);
	},
	reset : function() {
		for (var i = 0; i < 4; i++) {
			this.txtEnterIPParts[i].setValue('');
		}
	},
	setValue : function(value) {
		for (var i = 0; i < 4; i++) {
			this.txtEnterIPParts[i].setValue('');
		}
	},

	isValid : function() {
		var ipAddress = this.getValue();
		var bValid = isIpAddressValid(ipAddress, true);
		if (!bValid) {
			Ext.MessageBox.alert(_('Info'), _("Incorrect IP Address!"), null);
		}
		return bValid;
	},
	getValue : function() {
		return this.txtEnterIPParts[0].getValue() + "." + this.txtEnterIPParts[1].getValue() + "." + this.txtEnterIPParts[2].getValue() + "." + this.txtEnterIPParts[3].getValue();
	}
});